/*
 * Copyright (C) 2009, 2010, 2011, 2012, 2013  by CNRS and Grenoble-INP
 *
 * This file is part of LIBTLMPWT
 *
 * LIBTLMPWT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Authors: Matthieu Moy <matthieu.moy@imag.fr>, Grenoble-INP
 *          Tayeb Bouhadiba, CNRS
 *          Claude Helmstetter, CNRS
 *
 * This software has been partially supported by the French ANR project HELP
 *                                                         (ANR-09-SEGI-006)
 */

#include "gpio.hpp"
#include "gpio_regmap.h"
#include "bit.hpp"
#include <SDL/SDL.h>

using namespace sc_core;
using namespace std;

Gpio::Gpio(sc_core::sc_module_name):
  PwtModule(this),
  m_data(0), m_tri(0)
{
  INFO(2, "in constructor");
}

tlm::tlm_response_status
Gpio::read(const uint32_t& addr, uint32_t& data)
{
  Uint8 *keystate = SDL_GetKeyState(NULL);
  SDL_PumpEvents();
  switch (addr) {
  case GPIO_DATA_OFFSET:
    data = 0;
    if (keystate[SDLK_x]) {
      SET_BIT(data, GPIO_BTN0);
    }
    if (keystate[SDLK_c]) {
      SET_BIT(data, GPIO_BTN1);
    }
    if (keystate[SDLK_v]) {
      SET_BIT(data, GPIO_BTN2);
    }
    INFO(2, "Read GPIO_DATA_OFFSET = " << hex << data << dec);
    break;
  case GPIO_TRI_OFFSET:
    data = m_tri;
    break;
  default:
    WARNING("register not implemented");
    return tlm::TLM_ADDRESS_ERROR_RESPONSE;
  }
  return tlm::TLM_OK_RESPONSE;
}

tlm::tlm_response_status
Gpio::write(const uint32_t& addr, const uint32_t& data)
{
  switch (addr) {
  case GPIO_DATA_OFFSET:
    m_data = data;
    break;
  case GPIO_TRI_OFFSET:
    m_tri = data;
    break;
  default:
    WARNING("register not implemented");
    return tlm::TLM_ADDRESS_ERROR_RESPONSE;
  }
  return tlm::TLM_OK_RESPONSE;
}
