/*
 * Copyright (C) 2009, 2010, 2011, 2012, 2013  by CNRS and Grenoble-INP
 *
 * This file is part of LIBTLMPWT
 *
 * LIBTLMPWT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Authors: Matthieu Moy <matthieu.moy@imag.fr>, Grenoble-INP
 *          Tayeb Bouhadiba, CNRS
 *          Claude Helmstetter, CNRS
 *
 * This software has been partially supported by the French ANR project HELP
 *                                                         (ANR-09-SEGI-006)
 */

#ifndef _HEADLESS_VGA_HPP_
#define _HEADLESS_VGA_HPP_

#include "basic.hpp"
#include "host_memory.hpp"
#include "pwt_module.hpp"

class HeadlessVGA: public sc_core::sc_module, public PwtModule {
public:

  basic::initiator_socket<HeadlessVGA> initiator;
  basic::target_socket<HeadlessVGA> target;

  sc_core::sc_out<bool> irq;

  SC_HAS_PROCESS(HeadlessVGA);
  explicit HeadlessVGA(sc_core::sc_module_name);
  ~HeadlessVGA();

  tlm::tlm_response_status
  read(uint32_t addr, uint32_t& data);
  
  tlm::tlm_response_status
  write(uint32_t addr, uint32_t data);

protected:
  uint32_t m_address;
  bool m_intr;
  sc_core::sc_event address_event;
  HostMemory m_dmi;
  bool m_waiting;

  void vsync();
  void thread();

  virtual void draw();
  virtual void gui_draw(int x, int y, uint32_t data);
};

#endif // _HEADLESS_VGA_HPP_
