/*
 * Copyright (C) 2009, 2010, 2011, 2012, 2013  by CNRS and Grenoble-INP
 *
 * This file is part of LIBTLMPWT
 *
 * LIBTLMPWT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Authors: Matthieu Moy <matthieu.moy@imag.fr>, Grenoble-INP
 *          Tayeb Bouhadiba, CNRS
 *          Claude Helmstetter, CNRS
 *
 * This software has been partially supported by the French ANR project HELP
 *                                                         (ANR-09-SEGI-006)
 */

#include "top.hpp"
#include "parameter.hpp"
#include <fstream>

using namespace std;
using namespace sc_core;

//------------------------------------------------------------------------------
extern int qt_start(int argc, char *argv[], const sc_time &simtime,
                    AtmiWrapper *atmi_wrapper);

#ifdef ENABLE_QT

ParameterBool qt_enable("gui", "Launch the graphical temperature monitor (require -atmi)");
#define qt_enabled qt_enable.get()

#else // !ENABLE_QT

ParameterBool qt_enable("gui", "Fixed to FALSE at compile-time"
                        " (recompile with -DENABLE_QT)");
#define qt_enabled false

#endif//------------------------------------------------------------------------

ParameterInt simtime("sim-time",
                     "Duration of simualtions (in simualted ms)"
                     ", -1=infinite",
                     -1);

ParameterString dump_cfg_file("dump-cfg",
                              "Dump the configuration using the given file name",
                              "last.cfg");
                              
ParameterBool help("help", "Display help and exit");

int sc_main(int argc, char *argv[]) {
  cout << showbase;
  cerr << showbase;

  ParameterBase::parse(argc, argv);
  if (help.get()) {
    cout << "Following is the list of parameters." << endl
	 << "Parameters can be used in a configuration file or " << endl
	 << "prefixed with a - and use as command-line options." << endl;
    ParameterBase::dump(cout);
    return 0;
  }
  ofstream ofs;
  ofs.open(dump_cfg_file.get().c_str());
  if (!!ofs)
    ParameterBase::dump(ofs);
  ofs.close();

  Top top("TOP");

  if (!qt_enabled || top.atmi_wrapper==NULL) {
    if (simtime.get()>=0)
      sc_start(sc_time(simtime.get(),SC_MS));
    else
      sc_start();
    return 0;
  } else {
    return qt_start(argc, argv, sc_time(simtime.get(),SC_MS),
                    top.atmi_wrapper);
  }
}
