/*
 * Copyright (C) 2009, 2010, 2011, 2012, 2013  by CNRS and Grenoble-INP
 *
 * This file is part of LIBTLMPWT
 *
 * LIBTLMPWT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Authors: Matthieu Moy <matthieu.moy@imag.fr>, Grenoble-INP
 *          Tayeb Bouhadiba, CNRS
 *          Claude Helmstetter, CNRS
 *
 * This software has been partially supported by the French ANR project HELP
 *                                                         (ANR-09-SEGI-006)
 */

#ifndef _TOP_HPP_
#define _TOP_HPP_

#include "pwt_module.hpp"

// Forward declarations
namespace simsoc {
  class SimSoCBus;
  class ElfLoader;
}
class Memory;
class Uart;
class Timer;
class Generator;
class MBWrapper;
class FSLbus;
class Intc;
class HeadlessVGA;
class HwGoL;
class TempSensor;
class PowerCtrl;
class AtmiWrapper;

struct Top: sc_core::sc_module, public PwtModule {
  Top(sc_core::sc_module_name name);
  ~Top();

  // must be built before ATMI modules
  AtmiWrapper *atmi_wrapper;

  simsoc::SimSoCBus *bus;
  Memory *sram;
  Memory *inst_ram;
  Uart *uart;
  Timer *timer;
  MBWrapper *mb_wrapper;
  simsoc::ElfLoader *elf_loader;
  Intc *intc;
  HeadlessVGA *vga;
  HwGoL *hwgol;
  TempSensor *temp_sensor;
  PowerCtrl *power_ctrl;

  sc_core::sc_signal<bool> vga_sig;
  sc_core::sc_signal<bool> timer_sig;
  sc_core::sc_signal<bool> temp_sensor_sig;
  sc_core::sc_signal<bool> hwgol_sig;
  sc_core::sc_signal<bool> mb_irq_sig;

  // Microblaze processor wires
  sc_core::sc_signal<bool, sc_core::SC_UNCHECKED_WRITERS> sig_stub;
};

#endif // _TOP_HPP_
