/*
 * Copyright (C) 2009, 2010, 2011, 2012, 2013  by CNRS and Grenoble-INP
 *
 * This file is part of LIBTLMPWT
 *
 * LIBTLMPWT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Authors: Matthieu Moy <matthieu.moy@imag.fr>, Grenoble-INP
 *          Tayeb Bouhadiba, CNRS
 *          Claude Helmstetter, CNRS
 *
 * This software has been partially supported by the French ANR project HELP
 *                                                         (ANR-09-SEGI-006)
 */

#include "uart.hpp"
#include "uart_regmap.h"
#include "parameter.hpp"

using namespace std;
using namespace sc_core;

namespace {
  ParameterBool buffered("uart-buffer",
                     "Tell the TLM UART to buffer characters until end of line, so lines are printed one by one, with timestamp");

}

Uart::Uart(sc_module_name name):
  m_buffer("")
{}

tlm::tlm_response_status
Uart::write(uint32_t addr, uint32_t data)
{
  const char c = (char) data;
  switch (addr) {
  case UART_WRITE_CHAR:
    if (buffered.get()) {
      if (c=='\n') {
        m_buffer.push_back('\0');
        INFO(0, &m_buffer[0]);
        m_buffer.clear();
      } else {
        m_buffer.push_back(c);
      }
    } else {
      if (c=='\n') {
        cout << endl;
      } else {
        cout << c;
      }
    }
    break;
  case UART_WRITE_DEC:
    if (buffered.get()) {
      m_buffer += string_of((int)data);
    } else {
      cout << (int)data;
    }
    break;
  default:
    SC_REPORT_ERROR(name(), "write register not implemented");
    return tlm::TLM_ADDRESS_ERROR_RESPONSE;	
  }
  return tlm::TLM_OK_RESPONSE;
}

tlm::tlm_response_status
Uart::read(uint32_t, uint32_t&)
{
  SC_REPORT_ERROR(name(), "read not implemented");
  return tlm::TLM_ADDRESS_ERROR_RESPONSE;
}
