/*
 * Copyright (C) 2009, 2010, 2011, 2012, 2013  by CNRS and Grenoble-INP
 *
 * This file is part of LIBTLMPWT
 *
 * LIBTLMPWT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Authors: Matthieu Moy <matthieu.moy@imag.fr>, Grenoble-INP
 *          Tayeb Bouhadiba, CNRS
 *          Claude Helmstetter, CNRS
 *
 * This software has been partially supported by the French ANR project HELP
 *                                                         (ANR-09-SEGI-006)
 */

#include "vga.hpp"
#include "vga_regmap.h"
#include "bit.hpp"

using namespace sc_core;

namespace {
  int filter(const SDL_Event* event) {
    switch (event->type) {
    case SDL_QUIT:
      sc_stop();
      break;
    default:
      break;
    }
    return 0;
  }

  Uint16 black;
  Uint16 white;
}

Vga::Vga(sc_core::sc_module_name module_name):
  HeadlessVGA(module_name),
  screen(NULL)
{
  if (SDL_Init(SDL_INIT_VIDEO) < 0) {
    ERROR(SDL_GetError());
  }

  atexit(SDL_Quit);

  SDL_SetEventFilter(filter);
  
  screen = SDL_SetVideoMode(VGA_WIDTH, VGA_HEIGHT, 16,
                            SDL_DOUBLEBUF | SDL_HWSURFACE);

  if (screen->format->BytesPerPixel != 2) {
    ERROR(SDL_GetError());
  }

  black = SDL_MapRGB(screen->format, 0x00, 0x00, 0x00);
  white = SDL_MapRGB(screen->format, 0xFF, 0xFF, 0xFF);

  SDL_WM_SetCaption(sc_module::name(), NULL);
  INFO(2, "LCD controller TLM model, SDL Enabled");
}

void Vga::gui_draw(int x, int y, uint32_t d) {
  for (unsigned bit = 0; bit < 32; ++bit) {		
    Uint16 *bufp = (Uint16 *) screen->pixels +
      x + (y * screen->pitch / 2) + (32 - bit);
    
    if (TEST_BIT(d, bit)) {
      *bufp = white;
    } else {
      *bufp = black;
    }
  }
}

void Vga::draw() {
  SDL_PumpEvents();
  if (SDL_MUSTLOCK(screen)) {
    SDL_LockSurface(screen);
  }

  HeadlessVGA::draw();

  if (SDL_MUSTLOCK(screen)) {
    SDL_UnlockSurface(screen);
  }
  SDL_Flip(screen);
}
